@@ -1,7 +1,7 @@
 /* Test file for mpfr_exp.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 1999, 2001-2020 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -17,13 +17,9 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <limits.h>
-
 #include "mpfr-test.h"
 
 #ifdef CHECK_EXTERNAL
@@ -150,6 +146,22 @@ check_worst_cases (void)
       exit (1);
     }
 
+  mpfr_set_prec (x, 118);
+  mpfr_set_str_binary (x, "0.1110010100011101010000111110011000000000000000000000000000000000000000000000000000000000000000000000000000000000000000E-86");
+  mpfr_set_prec (y, 118);
+  mpfr_exp_2 (y, x, MPFR_RNDU);
+  mpfr_exp_3 (x, x, MPFR_RNDU);
+  if (mpfr_cmp (x, y))
+    {
+      printf ("mpfr_exp_2 and mpfr_exp_3 differ for prec=118\n");
+      printf ("mpfr_exp_2 gives ");
+      mpfr_out_str (stdout, 2, 0, y, MPFR_RNDN);
+      printf ("\nmpfr_exp_3 gives ");
+      mpfr_out_str (stdout, 2, 0, x, MPFR_RNDN);
+      printf ("\n");
+      exit (1);
+    }
+
   mpfr_clear (x);
   mpfr_clear (y);
   return 0;
@@ -180,14 +192,11 @@ compare_exp2_exp3 (mpfr_prec_t p0, mpfr_prec_t p1)
         {
           printf ("mpfr_exp_2 and mpfr_exp_3 disagree for rnd=%s and\nx=",
                   mpfr_print_rnd_mode (rnd));
-          mpfr_print_binary (x);
-          puts ("");
+          mpfr_dump (x);
           printf ("mpfr_exp_2 gives ");
-          mpfr_print_binary (y);
-          puts ("");
+          mpfr_dump (y);
           printf ("mpfr_exp_3 gives ");
-          mpfr_print_binary (z);
-          puts ("");
+          mpfr_dump (z);
           exit (1);
         }
   }
@@ -254,7 +263,7 @@ check_special (void)
   /* check exp(-inf) = +0 */
   mpfr_set_inf (x, -1);
   test_exp (y, x, MPFR_RNDN);
-  if (mpfr_cmp_ui (y, 0) || mpfr_sgn (y) < 0)
+  if (MPFR_NOTZERO (y) || MPFR_IS_NEG (y))
     {
       printf ("Error for exp(-inf)\n");
       exit (1);
@@ -345,11 +354,11 @@ check_special (void)
   set_emin (-10);
   mpfr_set_si (x, -9, MPFR_RNDN);
   test_exp (y, x, MPFR_RNDN);
-  if (mpfr_cmp_ui (y, 0) || mpfr_sgn (y) < 0)
+  if (MPFR_NOTZERO (y) || MPFR_IS_NEG (y))
     {
       printf ("Error for exp(-9) for emin=-10\n");
       printf ("Expected +0\n");
-      printf ("Got      "); mpfr_print_binary (y); puts ("");
+      printf ("Got      "); mpfr_dump (y);
       exit (1);
     }
   set_emin (emin);
@@ -534,55 +543,57 @@ overflowed_exp0 (void)
       /* and if emax < 0, 1 - eps is not representable either. */
       for (i = -1; i <= 1; i++)
         RND_LOOP (rnd)
-        {
-          mpfr_set_si_2exp (x, i, -512 * ABS (i), MPFR_RNDN);
-          mpfr_clear_flags ();
-          inex = mpfr_exp (x, x, (mpfr_rnd_t) rnd);
-          if ((i >= 0 || emax < 0 || rnd == MPFR_RNDN || rnd == MPFR_RNDU) &&
-              ! mpfr_overflow_p ())
-            {
-              printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
-                      "  The overflow flag is not set.\n",
-                      i, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
-              err = 1;
-            }
-          if (rnd == MPFR_RNDZ || rnd == MPFR_RNDD)
-            {
-              if (inex >= 0)
-                {
-                  printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
-                          "  The inexact value must be negative.\n",
-                          i, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
-                  err = 1;
-                }
-              if (! mpfr_equal_p (x, y))
-                {
-                  printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
-                          "  Got ", i, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
-                  mpfr_print_binary (x);
-                  printf (" instead of 0.11111111E%d.\n", emax);
-                  err = 1;
-                }
-            }
-          else
-            {
-              if (inex <= 0)
-                {
-                  printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
-                          "  The inexact value must be positive.\n",
-                          i, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
-                  err = 1;
-                }
-              if (! (mpfr_inf_p (x) && MPFR_SIGN (x) > 0))
-                {
-                  printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
-                          "  Got ", i, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
-                  mpfr_print_binary (x);
-                  printf (" instead of +Inf.\n");
-                  err = 1;
-                }
-            }
-        }
+          {
+            mpfr_set_si_2exp (x, i, -512 * ABS (i), MPFR_RNDN);
+            mpfr_clear_flags ();
+            inex = mpfr_exp (x, x, (mpfr_rnd_t) rnd);
+            if ((i >= 0 || emax < 0 || rnd == MPFR_RNDN || rnd == MPFR_RNDU) &&
+                ! mpfr_overflow_p ())
+              {
+                printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
+                        "  The overflow flag is not set.\n",
+                        i, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
+                err = 1;
+              }
+            if (rnd == MPFR_RNDZ || rnd == MPFR_RNDD)
+              {
+                if (inex >= 0)
+                  {
+                    printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
+                            "  The inexact value must be negative.\n",
+                            i, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
+                    err = 1;
+                  }
+                if (! mpfr_equal_p (x, y))
+                  {
+                    printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
+                            "  Got        ", i,
+                            mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
+                    mpfr_dump (x);
+                    printf ("  instead of 0.11111111E%d.\n", emax);
+                    err = 1;
+                  }
+              }
+            else if (rnd != MPFR_RNDF)
+              {
+                if (inex <= 0)
+                  {
+                    printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
+                            "  The inexact value must be positive.\n",
+                            i, mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
+                    err = 1;
+                  }
+                if (! (mpfr_inf_p (x) && MPFR_IS_POS (x)))
+                  {
+                    printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
+                            "  Got        ", i,
+                            mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
+                    mpfr_dump (x);
+                    printf ("  instead of +Inf.\n");
+                    err = 1;
+                  }
+              }
+          }
       set_emax (old_emax);
     }
 
@@ -607,6 +618,8 @@ bug20080731 (void)
   mpfr_set_str (x, "-2.c5c85fdf473de6af278ece700fcbdabd03cd0cb9ca62d8b62c@7",
                 16, MPFR_RNDN);
 
+  /* exp(x) is just below 0xf.fffffffffffffffp-1073741828 */
+
   mpfr_init2 (y1, prec);
   mpfr_exp (y1, x, MPFR_RNDU);
 
@@ -618,9 +631,9 @@ bug20080731 (void)
   if (mpfr_cmp0 (y1, y2) != 0)
     {
       printf ("Error in bug20080731\nExpected ");
-      mpfr_out_str (stdout, 16, 0, y2, MPFR_RNDN);
+      mpfr_dump (y2);
       printf ("\nGot      ");
-      mpfr_out_str (stdout, 16, 0, y1, MPFR_RNDN);
+      mpfr_dump (y1);
       printf ("\n");
       exit (1);
     }
@@ -708,7 +721,8 @@ underflow_up (int extended_emin)
                   mpfr_clear_flags ();
                   inex = e3 ? exp_3 (y, x, (mpfr_rnd_t) rnd)
                     : mpfr_exp (y, x, (mpfr_rnd_t) rnd);
-                  if (__gmpfr_flags != MPFR_FLAGS_INEXACT)
+                  /* for MPFR_RNDF, the inexact flag is undefined */
+                  if (__gmpfr_flags != MPFR_FLAGS_INEXACT && rnd != MPFR_RNDF)
                     {
                       printf ("Incorrect flags in underflow_up, eps > 0, %s",
                               mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
@@ -716,7 +730,10 @@ underflow_up (int extended_emin)
                         printf (" and extended emin");
                       printf ("\nfor precx = %d, precy = %d, %s\n",
                               precx, precy, e3 ? "mpfr_exp_3" : "mpfr_exp");
-                      printf ("Got %u instead of %u.\n", __gmpfr_flags,
+                      printf ("x="); mpfr_dump (x);
+                      printf ("y="); mpfr_dump (y);
+                      printf ("Got %u instead of %u.\n",
+                              (unsigned int) __gmpfr_flags,
                               (unsigned int) MPFR_FLAGS_INEXACT);
                       err = 1;
                     }
@@ -731,7 +748,9 @@ underflow_up (int extended_emin)
                       mpfr_dump (y);
                       err = 1;
                     }
-                  MPFR_ASSERTN (inex != 0);
+                  /* for MPFR_RNDF, the ternary value is undefined */
+                  if (rnd != MPFR_RNDF)
+                    MPFR_ASSERTN (inex != 0);
                   if (rnd == MPFR_RNDD || rnd == MPFR_RNDZ)
                     MPFR_ASSERTN (inex < 0);
                   if (rnd == MPFR_RNDU)
@@ -840,9 +859,12 @@ underflow_up (int extended_emin)
                                   precy + i, 1 - 2 * (precy + i));
                         printf (", %s\n", e3 ? "mpfr_exp_3" : "mpfr_exp");
                         printf ("Got %u instead of %u.\n",
-                                __gmpfr_flags, flags);
+                                (unsigned int) __gmpfr_flags, flags);
                         err = 1;
                       }
+                    if (rnd == MPFR_RNDF)
+                      continue; /* the test below makes no sense, since RNDF
+                                   does not give a deterministic result */
                     if (rnd == MPFR_RNDU || rnd == MPFR_RNDA || rnd == MPFR_RNDN ?
                         mpfr_cmp0 (y, minpos) != 0 : MPFR_NOTZERO (y))
                       {
@@ -952,6 +974,24 @@ underflow (void)
     }
 }
 
+/* bug found with GMP_CHECK_RANDOMIZE=1514290185 */
+static void
+bug20171223 (void)
+{
+  mpfr_t x, y;
+  int inex;
+
+  mpfr_init2 (x, 372);
+  mpfr_init2 (y, 2);
+  mpfr_set_str (x, "-6.9314716128384587678466323621915206417385796077947874471662159283492445979241549847386366371775938082803907383582e-01", 10, MPFR_RNDN);
+  /* exp(x) = 0.500000009638..., should be rounded to 0.5 */
+  inex = mpfr_exp (y, x, MPFR_RNDD);
+  MPFR_ASSERTN(mpfr_cmp_ui_2exp (y, 1, -1) == 0);
+  MPFR_ASSERTN(inex < 0);
+  mpfr_clear (x);
+  mpfr_clear (y);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -960,10 +1000,12 @@ main (int argc, char *argv[])
   if (argc > 1)
     check_large ();
 
+  bug20171223 ();
+
   check_inexact ();
   check_special ();
 
-  test_generic (2, 100, 100);
+  test_generic (MPFR_PREC_MIN, 100, 100);
 
   compare_exp2_exp3 (20, 1000);
   check_worst_cases();
